@@ -29,6 +29,7 @@ typedef struct
 {
 	GtkTextMark             *mark;
 	GtkSourceAssistantChild *child;
+	guint                    reposition_handler;
 } GtkSourceAssistantPrivate;
 
 static void buildable_iface_init (GtkBuildableIface *iface);
@@ -58,7 +59,7 @@ _gtk_source_assistant_hide_action (GtkWidget   *widget,
 {
 	g_assert (GTK_SOURCE_IS_ASSISTANT (widget));
 
-	gtk_popover_popdown (GTK_POPOVER (widget));
+	gtk_widget_hide (widget);
 }
 
 static void
@@ -103,15 +104,25 @@ _gtk_source_assistant_get_target_location (GtkSourceAssistant *assistant,
 	GTK_SOURCE_ASSISTANT_GET_CLASS (assistant)->get_target_location (assistant, location);
 }
 
-static void
+static int
+get_gutter_width (GtkSourceView *view)
+{
+	GtkSourceGutter *left = gtk_source_view_get_gutter (view, GTK_TEXT_WINDOW_LEFT);
+
+	if (left != NULL)
+	{
+		return gtk_widget_get_width (GTK_WIDGET (left));
+	}
+
+	return 0;
+}
+
+static gboolean
 _gtk_source_assistant_update_position (GtkSourceAssistant *assistant)
 {
 	GtkSourceAssistantPrivate *priv = _gtk_source_assistant_get_instance_private (assistant);
-	const GList *children = NULL;
 	GtkWidget *parent;
-	GdkRectangle rect;
-	int x = 0;
-	int y = 0;
+	gboolean changed = FALSE;
 
 	g_assert (GTK_SOURCE_IS_ASSISTANT (assistant));
 
@@ -119,37 +130,103 @@ _gtk_source_assistant_update_position (GtkSourceAssistant *assistant)
 
 	if (GTK_SOURCE_IS_VIEW (parent))
 	{
+		GdkRectangle visible_rect;
+		GdkRectangle old_rect;
+		GdkRectangle rect;
+		int old_x, old_y;
+		int x, y;
+
+		gtk_text_view_get_visible_rect (GTK_TEXT_VIEW (parent), &visible_rect);
+
 		_gtk_source_assistant_get_offset (assistant, &x, &y);
 		_gtk_source_assistant_get_target_location (assistant, &rect);
 
-		gtk_popover_set_offset (GTK_POPOVER (assistant), x, y);
-		gtk_popover_set_pointing_to (GTK_POPOVER (assistant), &rect);
-	}
+		rect.x -= visible_rect.x;
+		rect.y -= visible_rect.y;
+		rect.x += get_gutter_width (GTK_SOURCE_VIEW (parent));
 
-	gtk_popover_present (GTK_POPOVER (assistant));
+		gtk_popover_get_offset (GTK_POPOVER (assistant), &old_x, &old_y);
+
+		if (old_x != x || old_y != y)
+		{
+			gtk_popover_set_offset (GTK_POPOVER (assistant), x, y);
+			changed = TRUE;
+		}
+
+		if (!gtk_popover_get_pointing_to (GTK_POPOVER (assistant), &old_rect) ||
+		    !gdk_rectangle_equal (&old_rect, &rect))
+		{
+			gtk_popover_set_pointing_to (GTK_POPOVER (assistant), &rect);
+			changed = TRUE;
+		}
+	}
 
 	if (priv->child != NULL)
 	{
-		children = _gtk_source_assistant_child_get_attached (priv->child);
+		const GList *children = _gtk_source_assistant_child_get_attached (priv->child);
+
+		for (const GList *iter = children; iter; iter = iter->next)
+		{
+			GtkSourceAssistant *child = iter->data;
+			int x, y;
+
+			_gtk_source_assistant_get_offset (child, &x, &y);
+			gtk_popover_set_offset (GTK_POPOVER (child), x, y);
+		}
 	}
 
-	for (const GList *iter = children; iter; iter = iter->next)
-	{
-		GtkSourceAssistant *child = iter->data;
 
-		_gtk_source_assistant_get_offset (child, &x, &y);
-		gtk_popover_set_offset (GTK_POPOVER (child), x, y);
+	return changed;
+}
 
-		gtk_popover_present (GTK_POPOVER (child));
-	}
+static gboolean
+gtk_source_assistant_reposition_tick_cb (GtkWidget     *widget,
+                                         GdkFrameClock *frame_clock,
+                                         gpointer       user_data)
+{
+	GtkSourceAssistant *self = user_data;
+	GtkSourceAssistantPrivate *priv = _gtk_source_assistant_get_instance_private (self);
+
+	g_assert (GTK_SOURCE_IS_VIEW (widget));
+	g_assert (GDK_IS_FRAME_CLOCK (frame_clock));
+	g_assert (GTK_SOURCE_IS_ASSISTANT (self));
+
+	priv->reposition_handler = 0;
+
+	_gtk_source_assistant_update_position (self);
+
+	/* We have to gtk_widget_queue_resize() to ensure that the view will
+	 * continue to have an allocation when the assistant gets snapshotted.
+	 * gtk_widget_queue_allocate() is not enough to make that happen.
+	 * Without this, we often get flickering of the GtkSourceView.
+	 */
+	gtk_widget_queue_resize (widget);
+
+	return G_SOURCE_REMOVE;
 }
 
 void
-_gtk_source_assistant_reposition (GtkSourceAssistant *self)
+_gtk_source_assistant_set_needs_position (GtkSourceAssistant *self)
 {
+	GtkSourceAssistantPrivate *priv = _gtk_source_assistant_get_instance_private (self);
+
 	g_return_if_fail (GTK_SOURCE_IS_ASSISTANT (self));
 
-	_gtk_source_assistant_update_position (self);
+	if (priv->reposition_handler == 0)
+	{
+		GtkSourceView *view = _gtk_source_assistant_get_view (self);
+
+		if (view == NULL)
+		{
+			return;
+		}
+
+		priv->reposition_handler =
+			gtk_widget_add_tick_callback (GTK_WIDGET (view),
+			                              gtk_source_assistant_reposition_tick_cb,
+			                              g_object_ref (self),
+			                              g_object_unref);
+	}
 }
 
 static void
@@ -159,7 +236,7 @@ _gtk_source_assistant_show (GtkWidget *widget)
 
 	g_assert (GTK_SOURCE_IS_ASSISTANT (assistant));
 
-	_gtk_source_assistant_reposition (assistant);
+	_gtk_source_assistant_update_position (assistant);
 
 	GTK_WIDGET_CLASS (_gtk_source_assistant_parent_class)->show (widget);
 }
@@ -203,7 +280,7 @@ _gtk_source_assistant_real_get_offset (GtkSourceAssistant *assistant,
 	}
 	else
 	{
-		*y = (margin.bottom / 3 * 2) - 1;
+		*y = margin.bottom - 1;
 	}
 }
 
@@ -215,6 +292,19 @@ _gtk_source_assistant_dispose (GObject *object)
 
 	g_assert (GTK_SOURCE_IS_ASSISTANT (self));
 
+	if (priv->reposition_handler != 0)
+	{
+		GtkSourceView *view = _gtk_source_assistant_get_view (self);
+
+		if (view != NULL)
+		{
+			gtk_widget_remove_tick_callback (GTK_WIDGET (view),
+			                                 priv->reposition_handler);
+		}
+
+		priv->reposition_handler = 0;
+	}
+
 	_gtk_source_assistant_detach (self);
 	g_clear_object (&priv->mark);
 
@@ -236,7 +326,6 @@ _gtk_source_assistant_class_init (GtkSourceAssistantClass *klass)
 	klass->get_target_location = _gtk_source_assistant_real_get_target_location;
 
 	gtk_widget_class_install_action (widget_class, "assistant.hide", NULL, _gtk_source_assistant_hide_action);
-	gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Escape, 0, "assistant.hide", NULL);
 	gtk_widget_class_set_css_name (widget_class, "GtkSourceAssistant");
 }
 
