@@ -112,7 +112,7 @@
  * read the no-spell-check region).
  */
 
-#define UPDATE_BRACKET_DELAY          50
+#define UPDATE_BRACKET_DELAY_MSEC     50
 #define BRACKET_MATCHING_CHARS_LIMIT  10000
 #define CONTEXT_CLASSES_PREFIX        "gtksourceview:context-classes:"
 
@@ -163,6 +163,8 @@ typedef struct
 
 	gint64 insertion_count;
 
+	guint cursor_moved_block_count;
+
 	guint has_draw_spaces_tag : 1;
 	guint highlight_syntax : 1;
 	guint highlight_brackets : 1;
@@ -974,9 +976,28 @@ queue_bracket_highlighting_update (GtkSourceBuffer *buffer)
 {
 	GtkSourceBufferPrivate *priv = gtk_source_buffer_get_instance_private (buffer);
 
+	/* Short-circuit unless we're processing bracket highlighting. Since
+	 * gtk_source_buffer_set_highlight_matching_brackets() will call
+	 * update_bracket_highlighting() already, this short-circuit is safe
+	 * as we'll already be in stable state.
+	 */
+	if (priv->highlight_brackets == FALSE)
+	{
+		return;
+	}
+
+	/* Rearm existing GSource when possible */
 	if (priv->bracket_highlighting_timeout_id != 0)
 	{
-		g_source_remove (priv->bracket_highlighting_timeout_id);
+		GSource *source;
+		gint64 ready_time;
+
+		ready_time = g_get_monotonic_time () +
+			     (G_USEC_PER_SEC / 1000L) * UPDATE_BRACKET_DELAY_MSEC;
+
+		source = g_main_context_find_source_by_id (NULL, priv->bracket_highlighting_timeout_id);
+		g_source_set_ready_time (source, ready_time);
+		return;
 	}
 
 	/* Queue an update to the bracket location instead of doing it
@@ -998,7 +1019,7 @@ queue_bracket_highlighting_update (GtkSourceBuffer *buffer)
 	 */
 	priv->bracket_highlighting_timeout_id =
 		g_timeout_add_full (G_PRIORITY_LOW,
-		                    UPDATE_BRACKET_DELAY,
+		                    UPDATE_BRACKET_DELAY_MSEC,
 		                    bracket_highlighting_timeout_cb,
 		                    buffer,
 		                    NULL);
@@ -1011,6 +1032,13 @@ queue_bracket_highlighting_update (GtkSourceBuffer *buffer)
 static void
 cursor_moved (GtkSourceBuffer *buffer)
 {
+	GtkSourceBufferPrivate *priv = gtk_source_buffer_get_instance_private (buffer);
+
+	if (priv->cursor_moved_block_count > 0)
+	{
+		return;
+	}
+
 	queue_bracket_highlighting_update (buffer);
 
 	GTK_SOURCE_PROFILER_BEGIN_MARK;
@@ -3052,3 +3080,29 @@ _gtk_source_buffer_get_insertion_count (GtkSourceBuffer *buffer)
 
 	return priv->insertion_count;
 }
+
+void
+_gtk_source_buffer_block_cursor_moved (GtkSourceBuffer *buffer)
+{
+	GtkSourceBufferPrivate *priv = gtk_source_buffer_get_instance_private (buffer);
+
+	g_return_if_fail (GTK_SOURCE_IS_BUFFER (buffer));
+
+	priv->cursor_moved_block_count++;
+}
+
+void
+_gtk_source_buffer_unblock_cursor_moved (GtkSourceBuffer *buffer)
+{
+	GtkSourceBufferPrivate *priv = gtk_source_buffer_get_instance_private (buffer);
+
+	g_return_if_fail (GTK_SOURCE_IS_BUFFER (buffer));
+	g_return_if_fail (priv->cursor_moved_block_count > 0);
+
+	priv->cursor_moved_block_count--;
+
+	if (priv->cursor_moved_block_count == 0)
+	{
+		cursor_moved (buffer);
+	}
+}
