@@ -24,14 +24,12 @@
 
 #include "config.h"
 
-#include "gtksourcebindinggroup-private.h"
 #include "gtksourcecompletion-private.h"
 #include "gtksourcecompletioncontext-private.h"
 #include "gtksourcecompletionlist-private.h"
 #include "gtksourcecompletionproposal.h"
 #include "gtksourcecompletionprovider.h"
 #include "gtksourcebuffer.h"
-#include "gtksourcesignalgroup-private.h"
 #include "gtksourceview-private.h"
 
 /**
@@ -106,17 +104,17 @@ struct _GtkSourceCompletion
 	 * our current context. That includes handling notification of the first
 	 * result so that we can show the window, etc.
 	 */
-	GtkSourceSignalGroup *context_signals;
+	GSignalGroup *context_signals;
 
 	/* Signals to changes in the underlying GtkTextBuffer that we use to
 	 * determine where and how we can do completion.
 	 */
-	GtkSourceSignalGroup *buffer_signals;
+	GSignalGroup *buffer_signals;
 
 	/* We need to track various events on the view to ensure that we don't
 	 * activate at incorrect times.
 	 */
-	GtkSourceSignalGroup *view_signals;
+	GSignalGroup *view_signals;
 
 	/* The display popover for results */
 	GtkSourceCompletionList *display;
@@ -150,6 +148,12 @@ struct _GtkSourceCompletion
 	 */
 	guint page_size;
 
+	/* Handler for gtk_widget_add_tick_callback() to do delayed calls to
+	 * gtk_widget_hide() (so that we don't potentially flap between
+	 * hide/show while typing.
+	 */
+	guint hide_tick_handler;
+
 	/* If we're currently being displayed */
 	guint shown : 1;
 
@@ -197,29 +201,72 @@ static GParamSpec *properties [N_PROPS];
 static guint signals [N_SIGNALS];
 
 static void
-display_show (GtkSourceCompletionList *display)
+display_show (GtkSourceCompletion *self)
 {
-	g_assert (GTK_SOURCE_IS_COMPLETION_LIST (display));
+	g_assert (GTK_SOURCE_IS_COMPLETION (self));
 
-	gtk_widget_show (GTK_WIDGET (display));
-	gtk_widget_grab_focus (GTK_WIDGET (display));
+	if (self->hide_tick_handler != 0)
+	{
+		/* See display_hide() for why a tick handler is used */
+		gtk_widget_remove_tick_callback (GTK_WIDGET (self->view),
+		                                 self->hide_tick_handler);
+		self->hide_tick_handler = 0;
+	}
+
+	if (gtk_widget_get_mapped (GTK_WIDGET (self->view)))
+	{
+		gtk_widget_show (GTK_WIDGET (_gtk_source_completion_get_display (self)));
+	}
+}
+
+static gboolean
+display_hide_cb (GtkWidget     *widget,
+                 GdkFrameClock *frame_clock,
+                 gpointer       user_data)
+{
+	GtkSourceCompletion *self = user_data;
+
+	g_assert (GTK_SOURCE_IS_VIEW (widget));
+	g_assert (GDK_IS_FRAME_CLOCK (frame_clock));
+	g_assert (GTK_SOURCE_IS_COMPLETION (self));
+
+	self->hide_tick_handler = 0;
+
+	if (self->display != NULL)
+	{
+		gtk_widget_hide (GTK_WIDGET (self->display));
+	}
+
+	return G_SOURCE_REMOVE;
 }
 
 static void
-display_hide (GtkSourceCompletionList *display)
+display_hide (GtkSourceCompletion *self)
 {
-	GtkWidget *view;
+	g_assert (GTK_SOURCE_IS_COMPLETION (self));
 
-	g_assert (GTK_SOURCE_IS_COMPLETION_LIST (display));
+	/* We don't want to hide immediately because we might get another
+	 * change that causes the assistant to be redisplayed before the
+	 * next frame. Flapping the visibility is really distracting, so we'll
+	 * wait until the next start of the frame clock cycle to hide.
+	 *
+	 * We use the GtkSourceView's frame clock for this instead of the GtkPopover
+	 * because that is the frame clock we need to synchronize with to be sure
+	 * were not invalidating allocations during an active frame cycle.
+	 */
 
-	gtk_widget_hide (GTK_WIDGET (display));
-
-	view = gtk_widget_get_ancestor (GTK_WIDGET (display), GTK_SOURCE_TYPE_VIEW);
-
-	if (view != NULL)
+	if (self->display == NULL ||
+	    self->hide_tick_handler != 0 ||
+	    !gtk_widget_get_visible (GTK_WIDGET (self->display)))
 	{
-		gtk_widget_grab_focus (view);
+		return;
 	}
+
+	self->hide_tick_handler =
+		gtk_widget_add_tick_callback (GTK_WIDGET (self->view),
+		                              display_hide_cb,
+		                              g_object_ref (self),
+		                              g_object_unref);
 }
 
 static gboolean
@@ -302,7 +349,6 @@ gtk_source_completion_complete_cb (GObject      *object,
                                    gpointer      user_data)
 {
 	GtkSourceCompletionContext *context = (GtkSourceCompletionContext *)object;
-	GtkSourceCompletionList *list;
 	GtkSourceCompletion *self = user_data;
 	GError *error = NULL;
 
@@ -335,12 +381,14 @@ gtk_source_completion_complete_cb (GObject      *object,
 		_gtk_source_completion_context_refilter (context);
 	}
 
-	list = _gtk_source_completion_get_display (self);
-
 	if (!gtk_source_completion_context_get_empty (context))
-		display_show (list);
+	{
+		display_show (self);
+	}
 	else
-		display_hide (list);
+	{
+		display_hide (self);
+	}
 
 cleanup:
 	g_clear_error (&error);
@@ -357,7 +405,7 @@ _gtk_source_completion_set_context (GtkSourceCompletion        *self,
 	if (g_set_object (&self->context, context))
 	{
 		g_clear_handle_id (&self->queued_update, g_source_remove);
-		gtk_source_signal_group_set_target (self->context_signals, context);
+		g_signal_group_set_target (self->context_signals, context);
 	}
 }
 
@@ -428,7 +476,8 @@ gtk_source_completion_compute_bounds (GtkSourceCompletion *self,
 
 static void
 gtk_source_completion_start (GtkSourceCompletion           *self,
-                             GtkSourceCompletionActivation  activation)
+                             GtkSourceCompletionActivation  activation,
+                             gboolean                       from_trigger)
 {
 	GtkSourceCompletionContext *context = NULL;
 	GtkTextBuffer *buffer;
@@ -442,8 +491,11 @@ gtk_source_completion_start (GtkSourceCompletion           *self,
 
 	if (!gtk_source_completion_compute_bounds (self, &begin, &end))
 	{
-		if (activation == GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE)
+		if (!from_trigger && activation == GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE)
+		{
 			goto cleanup;
+		}
+
 		begin = end;
 	}
 
@@ -471,9 +523,13 @@ gtk_source_completion_start (GtkSourceCompletion           *self,
 		_gtk_source_completion_list_set_context (self->display, context);
 
 		if (!gtk_source_completion_context_get_empty (context))
-			display_show (self->display);
+		{
+			display_show (self);
+		}
 		else
-			display_hide (self->display);
+		{
+			display_hide (self);
+		}
 	}
 
 cleanup:
@@ -482,7 +538,8 @@ cleanup:
 
 static void
 gtk_source_completion_update (GtkSourceCompletion           *self,
-                              GtkSourceCompletionActivation  activation)
+                              GtkSourceCompletionActivation  activation,
+                              gboolean                       from_trigger)
 {
 	GtkTextBuffer *buffer;
 	GtkTextMark *insert;
@@ -503,8 +560,6 @@ gtk_source_completion_update (GtkSourceCompletion           *self,
 
 	if (_gtk_source_completion_context_can_refilter (self->context, &begin, &end))
 	{
-		GtkSourceCompletionList *display = _gtk_source_completion_get_display (self);
-
 		/*
 		 * Make sure we update providers that have already delivered results
 		 * even though some of them won't be ready yet.
@@ -522,9 +577,13 @@ gtk_source_completion_update (GtkSourceCompletion           *self,
 		}
 
 		if (!gtk_source_completion_context_get_empty (self->context))
-			display_show (display);
+		{
+			display_show (self);
+		}
 		else
-			display_hide (display);
+		{
+			display_hide (self);
+		}
 
 		return;
 	}
@@ -565,7 +624,7 @@ gtk_source_completion_update (GtkSourceCompletion           *self,
 
 reset:
 	gtk_source_completion_cancel (self);
-	gtk_source_completion_start (self, activation);
+	gtk_source_completion_start (self, activation, from_trigger);
 }
 
 static void
@@ -590,34 +649,40 @@ gtk_source_completion_real_show (GtkSourceCompletion *self)
 
 	if (self->context == NULL)
 	{
-		gtk_source_completion_start (self, GTK_SOURCE_COMPLETION_ACTIVATION_USER_REQUESTED);
+		gtk_source_completion_start (self, GTK_SOURCE_COMPLETION_ACTIVATION_USER_REQUESTED, FALSE);
 	}
 	else
 	{
-		gtk_source_completion_update (self, GTK_SOURCE_COMPLETION_ACTIVATION_USER_REQUESTED);
+		gtk_source_completion_update (self, GTK_SOURCE_COMPLETION_ACTIVATION_USER_REQUESTED, FALSE);
 	}
 
 	_gtk_source_completion_list_set_context (display, self->context);
 
 	if (!gtk_source_completion_context_get_empty (self->context))
-		display_show (display);
+	{
+		display_show (self);
+	}
 	else
-		display_hide (display);
+	{
+		display_hide (self);
+	}
 }
 
 static gboolean
 gtk_source_completion_queued_update_cb (gpointer user_data)
 {
-  GtkSourceCompletion *self = user_data;
+	GtkSourceCompletion *self = user_data;
 
-  g_assert (GTK_SOURCE_IS_COMPLETION (self));
+	g_assert (GTK_SOURCE_IS_COMPLETION (self));
 
-  self->queued_update = 0;
+	self->queued_update = 0;
 
-  if (self->context != NULL)
-    gtk_source_completion_update (self, GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE);
+	if (self->context != NULL)
+	{
+		gtk_source_completion_update (self, GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE, FALSE);
+	}
 
-  return G_SOURCE_REMOVE;
+	return G_SOURCE_REMOVE;
 }
 
 static void
@@ -651,17 +716,23 @@ gtk_source_completion_notify_context_empty_cb (GtkSourceCompletion        *self,
 	g_assert (GTK_SOURCE_IS_COMPLETION_CONTEXT (context));
 
 	if (context != self->context)
+	{
+		/* Delayed notification from a context we no longer care about.
+		 * Just silently drop the notification.
+		 */
 		return;
+	}
 
 	if (gtk_source_completion_context_get_empty (context))
 	{
 		if (self->display != NULL)
-			display_hide (self->display);
+		{
+			display_hide (self);
+		}
 	}
 	else
 	{
-		GtkSourceCompletionList *display = _gtk_source_completion_get_display (self);
-		display_show (display);
+		display_show (self);
 	}
 }
 
@@ -742,7 +813,8 @@ gtk_source_completion_buffer_insert_text_after_cb (GtkSourceCompletion *self,
                                                    gint                 len,
                                                    GtkTextBuffer       *buffer)
 {
-	GtkSourceCompletionActivation activation = GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE;
+	const GtkSourceCompletionActivation activation = GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE;
+	gboolean from_trigger = FALSE;
 	GtkTextIter begin;
 	GtkTextIter end;
 
@@ -779,8 +851,8 @@ gtk_source_completion_buffer_insert_text_after_cb (GtkSourceCompletion *self,
 					 * completion. We need to cancel the previous completion (if any) first
 					 * and then try to start a new completion due to trigger.
 					 */
+					from_trigger = TRUE;
 					gtk_source_completion_cancel (self);
-					activation = GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE;
 					goto do_completion;
 				}
 			}
@@ -793,9 +865,9 @@ gtk_source_completion_buffer_insert_text_after_cb (GtkSourceCompletion *self,
 do_completion:
 
 	if (self->context == NULL)
-		gtk_source_completion_start (self, activation);
+		gtk_source_completion_start (self, activation, from_trigger);
 	else
-		gtk_source_completion_update (self, activation);
+		gtk_source_completion_update (self, activation, from_trigger);
 }
 
 static void
@@ -834,7 +906,7 @@ gtk_source_completion_set_view (GtkSourceCompletion *self,
 
 	if (g_set_weak_pointer (&self->view, view))
 	{
-		gtk_source_signal_group_set_target (self->view_signals, view);
+		g_signal_group_set_target (self->view_signals, view);
 		g_object_bind_property (view, "buffer",
 		                        self->buffer_signals, "target",
 		                        G_BINDING_SYNC_CREATE);
@@ -842,15 +914,15 @@ gtk_source_completion_set_view (GtkSourceCompletion *self,
 }
 
 static void
-on_buffer_signals_bind (GtkSourceCompletion  *self,
-                        GtkSourceBuffer      *buffer,
-                        GtkSourceSignalGroup *signals_)
+on_buffer_signals_bind (GtkSourceCompletion *self,
+                        GtkSourceBuffer     *buffer,
+                        GSignalGroup        *signals_)
 {
 	GtkTextIter where;
 
 	g_assert (GTK_SOURCE_IS_COMPLETION (self));
 	g_assert (GTK_SOURCE_IS_BUFFER (buffer));
-	g_assert (GTK_SOURCE_IS_SIGNAL_GROUP (signals_));
+	g_assert (G_IS_SIGNAL_GROUP (signals_));
 
 	if (self->disposed)
 		return;
@@ -877,9 +949,20 @@ gtk_source_completion_dispose (GObject *object)
 
 	self->disposed = TRUE;
 
-	gtk_source_signal_group_set_target (self->context_signals, NULL);
-	gtk_source_signal_group_set_target (self->buffer_signals, NULL);
-	gtk_source_signal_group_set_target (self->view_signals, NULL);
+	if (self->hide_tick_handler != 0)
+	{
+		if (self->view != NULL)
+		{
+			gtk_widget_remove_tick_callback (GTK_WIDGET (self->view),
+			                                 self->hide_tick_handler);
+		}
+
+		self->hide_tick_handler = 0;
+	}
+
+	g_signal_group_set_target (self->context_signals, NULL);
+	g_signal_group_set_target (self->buffer_signals, NULL);
+	g_signal_group_set_target (self->view_signals, NULL);
 
 	g_clear_pointer ((GtkSourceAssistant **)&self->display, _gtk_source_assistant_destroy);
 
@@ -914,6 +997,10 @@ gtk_source_completion_get_property (GObject    *object,
 
 	switch (prop_id)
 	{
+	case PROP_PAGE_SIZE:
+		g_value_set_uint (value, gtk_source_completion_get_page_size (self));
+		break;
+
 	case PROP_REMEMBER_INFO_VISIBILITY:
 		g_value_set_boolean (value, self->remember_info_visibility);
 		break;
@@ -945,6 +1032,10 @@ gtk_source_completion_set_property (GObject      *object,
 
 	switch (prop_id)
 	{
+	case PROP_PAGE_SIZE:
+		gtk_source_completion_set_page_size (self, g_value_get_uint (value));
+		break;
+
 	case PROP_REMEMBER_INFO_VISIBILITY:
 		self->remember_info_visibility = g_value_get_boolean (value);
 		if (self->display != NULL)
@@ -1148,9 +1239,9 @@ gtk_source_completion_init (GtkSourceCompletion *self)
 {
 	self->cancellable = g_cancellable_new ();
 	self->providers = g_ptr_array_new_with_free_func (g_object_unref);
-	self->buffer_signals = gtk_source_signal_group_new (GTK_TYPE_TEXT_BUFFER);
-	self->context_signals = gtk_source_signal_group_new (GTK_SOURCE_TYPE_COMPLETION_CONTEXT);
-	self->view_signals = gtk_source_signal_group_new (GTK_SOURCE_TYPE_VIEW);
+	self->buffer_signals = g_signal_group_new (GTK_TYPE_TEXT_BUFFER);
+	self->context_signals = g_signal_group_new (GTK_SOURCE_TYPE_COMPLETION_CONTEXT);
+	self->view_signals = g_signal_group_new (GTK_SOURCE_TYPE_VIEW);
 	self->page_size = DEFAULT_PAGE_SIZE;
 	self->show_icons = TRUE;
 
@@ -1159,11 +1250,11 @@ gtk_source_completion_init (GtkSourceCompletion *self)
 	 * having results (or vice-versa, when we've filtered to the point of
 	 * no results).
 	 */
-	gtk_source_signal_group_connect_object (self->context_signals,
-	                                        "notify::empty",
-	                                        G_CALLBACK (gtk_source_completion_notify_context_empty_cb),
-	                                        self,
-	                                        G_CONNECT_SWAPPED);
+	g_signal_group_connect_object (self->context_signals,
+	                               "notify::empty",
+	                               G_CALLBACK (gtk_source_completion_notify_context_empty_cb),
+	                               self,
+	                               G_CONNECT_SWAPPED);
 
 	/*
 	 * We need to know when the buffer inserts or deletes text so that we
@@ -1175,41 +1266,41 @@ gtk_source_completion_init (GtkSourceCompletion *self)
 	                         G_CALLBACK (on_buffer_signals_bind),
 	                         self,
 	                         G_CONNECT_SWAPPED);
-	gtk_source_signal_group_connect_object (self->buffer_signals,
-	                                        "delete-range",
-	                                        G_CALLBACK (gtk_source_completion_buffer_delete_range_after_cb),
-	                                        self,
-	                                        G_CONNECT_AFTER | G_CONNECT_SWAPPED);
-	gtk_source_signal_group_connect_object (self->buffer_signals,
-	                                        "insert-text",
-	                                        G_CALLBACK (gtk_source_completion_buffer_insert_text_after_cb),
-	                                        self,
-	                                        G_CONNECT_AFTER | G_CONNECT_SWAPPED);
-	gtk_source_signal_group_connect_object (self->buffer_signals,
-	                                        "mark-set",
-	                                        G_CALLBACK (gtk_source_completion_buffer_mark_set_cb),
-	                                        self,
-	                                        G_CONNECT_SWAPPED);
+	g_signal_group_connect_object (self->buffer_signals,
+	                               "delete-range",
+	                               G_CALLBACK (gtk_source_completion_buffer_delete_range_after_cb),
+	                               self,
+	                               G_CONNECT_AFTER | G_CONNECT_SWAPPED);
+	g_signal_group_connect_object (self->buffer_signals,
+	                               "insert-text",
+	                               G_CALLBACK (gtk_source_completion_buffer_insert_text_after_cb),
+	                               self,
+	                               G_CONNECT_AFTER | G_CONNECT_SWAPPED);
+	g_signal_group_connect_object (self->buffer_signals,
+	                               "mark-set",
+	                               G_CALLBACK (gtk_source_completion_buffer_mark_set_cb),
+	                               self,
+	                               G_CONNECT_SWAPPED);
 
 	/*
 	 * We track some events on the view that owns our GtkSourceCompletion instance so
 	 * that we can hide the window when it definitely should not be displayed.
 	 */
-	gtk_source_signal_group_connect_object (self->view_signals,
-	                                        "move-cursor",
-	                                        G_CALLBACK (gtk_source_completion_view_move_cursor_cb),
-	                                        self,
-	                                        G_CONNECT_AFTER | G_CONNECT_SWAPPED);
-	gtk_source_signal_group_connect_object (self->view_signals,
-	                                        "paste-clipboard",
-	                                        G_CALLBACK (gtk_source_completion_block_interactive),
-	                                        self,
-	                                        G_CONNECT_SWAPPED);
-	gtk_source_signal_group_connect_object (self->view_signals,
-	                                        "paste-clipboard",
-	                                        G_CALLBACK (gtk_source_completion_unblock_interactive),
-	                                        self,
-	                                        G_CONNECT_AFTER | G_CONNECT_SWAPPED);
+	g_signal_group_connect_object (self->view_signals,
+	                               "move-cursor",
+	                               G_CALLBACK (gtk_source_completion_view_move_cursor_cb),
+	                               self,
+	                               G_CONNECT_AFTER | G_CONNECT_SWAPPED);
+	g_signal_group_connect_object (self->view_signals,
+	                               "paste-clipboard",
+	                               G_CALLBACK (gtk_source_completion_block_interactive),
+	                               self,
+	                               G_CONNECT_SWAPPED);
+	g_signal_group_connect_object (self->view_signals,
+	                               "paste-clipboard",
+	                               G_CALLBACK (gtk_source_completion_unblock_interactive),
+	                               self,
+	                               G_CONNECT_AFTER | G_CONNECT_SWAPPED);
 }
 
 /**
@@ -1424,7 +1515,7 @@ _gtk_source_completion_get_display (GtkSourceCompletion *self)
  * @priority: (out) (allow-none): An optional location for the score of the match
  *
  * This helper function can do a fuzzy match for you giving a haystack and
- * casefolded needle. 
+ * casefolded needle.
  *
  * Casefold your needle using [func@GLib.utf8_casefold] before
  * running the query.
@@ -1514,7 +1605,7 @@ add_attributes (PangoAttrList **attrs,
 		*attrs = pango_attr_list_new ();
 	}
 
-	attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE);
+	attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE_LINE);
 	attr->start_index = begin;
 	attr->end_index = end;
 	pango_attr_list_insert (*attrs, g_steal_pointer (&attr));
