@@ -24,7 +24,6 @@
 #include "gtksourcegutter-private.h"
 #include "gtksourcegutterlines.h"
 #include "gtksourcegutterlines-private.h"
-#include "gtksourcesignalgroup-private.h"
 #include "gtksourcestylescheme-private.h"
 #include "gtksourceview-private.h"
 #include "gtksourcegutterrenderer.h"
@@ -74,13 +73,17 @@ struct _GtkSourceGutter
 	GtkSourceView        *view;
 	GList                *renderers;
 	GtkSourceGutterLines *lines;
-	GtkSourceSignalGroup *signals;
+	GSignalGroup         *signals;
 	GBinding             *target_binding;
 
 	GtkTextWindowType     window_type;
 	GtkOrientation        orientation;
 
+	double                pointer_x;
+	double                pointer_y;
+
 	guint                 is_drawing : 1;
+	guint                 pointer_in_gutter : 1;
 };
 
 G_DEFINE_TYPE (GtkSourceGutter, gtk_source_gutter, GTK_TYPE_WIDGET)
@@ -320,10 +323,60 @@ gtk_source_gutter_measure (GtkWidget      *widget,
 }
 
 static void
-gtk_source_gutter_set_property (GObject       *object,
-                                guint          prop_id,
-                                const GValue  *value,
-                                GParamSpec    *pspec)
+gtk_source_gutter_motion_cb (GtkSourceGutter          *gutter,
+			     double                    x,
+			     double                    y,
+			     GtkEventControllerMotion *motion)
+{
+	g_assert (GTK_SOURCE_IS_GUTTER (gutter));
+	g_assert (GTK_IS_EVENT_CONTROLLER_MOTION (motion));
+
+	gutter->pointer_x = x;
+	gutter->pointer_y = y;
+	gutter->pointer_in_gutter = TRUE;
+
+	_gtk_source_gutter_queue_draw (gutter);
+}
+
+static void
+gtk_source_gutter_leave_cb (GtkSourceGutter          *gutter,
+			    GtkEventControllerMotion *motion)
+{
+	g_assert (GTK_SOURCE_IS_GUTTER (gutter));
+	g_assert (GTK_IS_EVENT_CONTROLLER_MOTION (motion));
+
+	gutter->pointer_x = -1;
+	gutter->pointer_y = -1;
+	gutter->pointer_in_gutter = FALSE;
+
+	_gtk_source_gutter_queue_draw (gutter);
+}
+
+static void
+gtk_source_gutter_root (GtkWidget *widget)
+{
+	GtkWidget *parent;
+
+	g_assert (GTK_SOURCE_IS_GUTTER (widget));
+
+	GTK_WIDGET_CLASS (gtk_source_gutter_parent_class)->root (widget);
+
+	parent = gtk_widget_get_parent (widget);
+
+	/* The GtkTextViewChild has "overflow" set to Hidden and we
+	 * want to allow drawing over that.
+	 */
+	if (parent != NULL)
+	{
+		gtk_widget_set_overflow (parent, GTK_OVERFLOW_VISIBLE);
+	}
+}
+
+static void
+gtk_source_gutter_set_property (GObject      *object,
+                                guint         prop_id,
+                                const GValue *value,
+                                GParamSpec   *pspec)
 {
 	GtkSourceGutter *gutter = GTK_SOURCE_GUTTER (object);
 
@@ -372,7 +425,7 @@ gtk_source_gutter_dispose (GObject *object)
 
 	if (gutter->signals != NULL)
 	{
-		gtk_source_signal_group_set_target (gutter->signals, NULL);
+		g_signal_group_set_target (gutter->signals, NULL);
 		g_clear_object (&gutter->signals);
 	}
 
@@ -394,6 +447,7 @@ gtk_source_gutter_class_init (GtkSourceGutterClass *klass)
 	widget_class->measure = gtk_source_gutter_measure;
 	widget_class->size_allocate = gtk_source_gutter_size_allocate;
 	widget_class->snapshot = gtk_source_gutter_snapshot;
+	widget_class->root = gtk_source_gutter_root;
 
 	/**
 	 * GtkSourceGutter:view:
@@ -433,17 +487,17 @@ gtk_source_gutter_init (GtkSourceGutter *gutter)
 
 	gutter->window_type = GTK_TEXT_WINDOW_LEFT;
 
-	gutter->signals = gtk_source_signal_group_new (GTK_TYPE_ADJUSTMENT);
-	gtk_source_signal_group_connect_object (gutter->signals,
-						"value-changed",
-						G_CALLBACK (on_adjustment_value_changed),
-						gutter,
-						0);
-	gtk_source_signal_group_connect_object (gutter->signals,
-						"notify::upper",
-						G_CALLBACK (on_adjustment_upper_changed),
-						gutter,
-						0);
+	gutter->signals = g_signal_group_new (GTK_TYPE_ADJUSTMENT);
+	g_signal_group_connect_object (gutter->signals,
+	                               "value-changed",
+	                               G_CALLBACK (on_adjustment_value_changed),
+	                               gutter,
+	                               0);
+	g_signal_group_connect_object (gutter->signals,
+	                               "notify::upper",
+	                               G_CALLBACK (on_adjustment_upper_changed),
+	                               gutter,
+	                               0);
 
 	/* Setup fallback click handling */
 	click = gtk_gesture_click_new ();
@@ -457,15 +511,15 @@ gtk_source_gutter_init (GtkSourceGutter *gutter)
 	motion = gtk_event_controller_motion_new ();
 	g_signal_connect_swapped (motion,
 	                          "enter",
-	                          G_CALLBACK (gtk_widget_queue_draw),
+	                          G_CALLBACK (gtk_source_gutter_motion_cb),
 	                          gutter);
 	g_signal_connect_swapped (motion,
 	                          "leave",
-	                          G_CALLBACK (gtk_widget_queue_draw),
+	                          G_CALLBACK (gtk_source_gutter_leave_cb),
 	                          gutter);
 	g_signal_connect_swapped (motion,
 	                          "motion",
-	                          G_CALLBACK (gtk_widget_queue_draw),
+	                          G_CALLBACK (gtk_source_gutter_motion_cb),
 	                          gutter);
 	gtk_widget_add_controller (GTK_WIDGET (gutter), motion);
 }
@@ -705,6 +759,7 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 	GtkTextIter cur, sel;
 	gboolean needs_wrap_first = FALSE;
 	gboolean needs_wrap_last = FALSE;
+	int clip_width = 0;
 
 	g_clear_object (&gutter->lines);
 
@@ -742,6 +797,27 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 	                                              needs_wrap_first,
 	                                              needs_wrap_last);
 
+	/* Get the line under the pointer so we can set "prelit" on it */
+	if (gutter->pointer_in_gutter)
+	{
+		GtkTextIter pointer;
+		GdkRectangle pointer_rect;
+
+		gtk_text_view_get_iter_at_location (text_view,
+		                                    &pointer,
+		                                    0,
+		                                    visible_rect.y + gutter->pointer_y);
+		gtk_text_view_get_iter_location (text_view, &pointer, &pointer_rect);
+		pointer_rect.y -= visible_rect.y;
+
+		if (gutter->pointer_y >= pointer_rect.y &&
+		    gutter->pointer_y <= pointer_rect.y + pointer_rect.height)
+		{
+			guint line = gtk_text_iter_get_line (&pointer);
+			gtk_source_gutter_lines_add_class (gutter->lines, line, "prelit");
+		}
+	}
+
 	/* Draw the current-line highlight if necessary */
 	if (gtk_source_view_get_highlight_current_line (gutter->view) &&
 	    !gtk_text_buffer_get_selection_bounds (buffer, &cur, &sel))
@@ -790,10 +866,18 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 		                                   gutter->lines);
 	}
 
+	clip_width = gtk_widget_get_width (widget);
+
+	/* Allow drawing over the left margin from renderers */
+	if (gutter->window_type == GTK_TEXT_WINDOW_LEFT)
+	{
+		clip_width += gtk_text_view_get_left_margin (text_view);
+	}
+
 	gtk_snapshot_push_clip (snapshot,
 	                        &GRAPHENE_RECT_INIT (0,
 	                                             0,
-	                                             gtk_widget_get_width (widget),
+						     clip_width,
 	                                             gtk_widget_get_height (widget)));
 
 	/* Now let the renderers draw the content for each line. Because
