@@ -25,7 +25,6 @@
 
 #include "gtksourceinformative-private.h"
 #include "gtksourceassistant-private.h"
-#include "gtksourcesignalgroup-private.h"
 
 G_BEGIN_DECLS
 
@@ -39,7 +38,7 @@ typedef struct _GtkSourceViewSnippets
 {
 	GtkSourceView        *view;
 	GtkSourceBuffer      *buffer;
-	GtkSourceSignalGroup *snippet_signals;
+	GSignalGroup         *snippet_signals;
 	GtkSourceInformative *informative;
 	GQueue                queue;
 	gulong                buffer_insert_text_handler;
@@ -49,6 +48,7 @@ typedef struct _GtkSourceViewSnippets
 	gulong                buffer_cursor_moved_handler;
 } GtkSourceViewSnippets;
 
+gboolean _gtk_source_view_has_snippet              (GtkSourceView           *view);
 void     _gtk_source_view_add_assistant            (GtkSourceView           *view,
                                                     GtkSourceAssistant      *assistant);
 void     _gtk_source_view_remove_assistant         (GtkSourceView           *view,
@@ -61,6 +61,7 @@ void     _gtk_source_view_assistants_remove        (GtkSourceViewAssistants *ass
                                                     GtkSourceAssistant      *assistant);
 void     _gtk_source_view_assistants_remove        (GtkSourceViewAssistants *assistants,
                                                     GtkSourceAssistant      *assistant);
+gboolean _gtk_source_view_assistants_hide_all      (GtkSourceViewAssistants *assistants);
 void     _gtk_source_view_assistants_shutdown      (GtkSourceViewAssistants *assistants);
 void     _gtk_source_view_assistants_size_allocate (GtkSourceViewAssistants *assistants,
                                                     int                      width,
